///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  FastDataSource.h                                                                  //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
//<LIBTAG:3655f03d-8d45-4bd6-883d-66d7ab996593>

#pragma once

#include  <fstream>
#include  <chrono>
#include  <iostream>
#include  <iomanip>
#include  "RTIF.h"
#include  "FifoWriterTool.h"

using namespace std;
using namespace chrono;

// For debug prints
extern  mutex  DebugMutex;     // Global Debug Mutex


class {VAL label} : public RTIF
{
public:

  const bool  DebugPrints = false;

  // Parameters
  const bool      RealtimeDisplayFlag = { VAL params/realtime_display };
  const double    RealtimeDisplayUpdatePeriod = { VAL params/realtime_display_update_rate_int_secs };

  uint64_t  ColumnsRead;

  // Timing information
  bool                               TimerStarted;
  high_resolution_clock::time_point  StartTime;
  high_resolution_clock::time_point  EndTime;
  uint64_t                           NumBatchesProcessed;

public:

  // Threading and synchronization
  thread  ControlThread;          // Thread handle for controller thread

  // Optional real=time display thread
  thread  RealTimeDisplayThread;

  // Local control mutex (non-CV)
  mutex   ControlMutex;

  // In FIFO
  const vector<uint32_t>  {VAL params/out_fifo_name}_Dimensions;
  const vector<bool>      {VAL params/out_fifo_name}_WrapFlags;
  BufMap<{VAL params/data_type},{VAL params/alloc_data_type}>  {VAL params/out_fifo_name}_Extent;

  FIFO<{VAL params/data_type},{VAL params/alloc_data_type}>            {VAL params/out_fifo_name};
  FifoWriterTool<{VAL params/data_type},{VAL params/alloc_data_type}>  FWT;

public:

  // Standard definitions
  {VAL label}(void);

  RTIF::ExitStateType  Init_Allocate(void);    // Called once to allocate dynamic memory
  RTIF::ExitStateType  Init_Reset(void);       // Called any time to reset all counters to their initial state
  RTIF::ExitStateType  KillThreads(void);      // Called to kill all threads

  // Display
  string  Display_Constructor(void);
  string  Display_InitAlloc(void);
  string  Display_InitReset(void);
  string  Display_Kill(void);

private:

  // Controller for the main processing threads
  static RTIF::ExitStateType   ProcessBatches( {VAL label}& _Obj );   // Called to process a batch of data

  // Optional Real-time Display Tread
  static RTIF::ExitStateType  RealtimeDisplay( {VAL label}& _Obj );
};


