///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  SubSpaceFilter.h                                                                  //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
//<LIBTAG:4bc4bb5c-f814-4d5f-9d91-5b7fb92525d7>

#pragma once

#include  <fftw3.h>
#include  "AlignedMemAlloc.h"
#include  "ComplexHermitionVector.h"
#include  "RTIF.h"
#include  "FifoReaderTool.h"
#include  "FifoWriterTool.h"
#include  "{VAL label}.h"


// For debug prints
extern  mutex  DebugMutex;     // Global Debug Mutex


class {VAL label} : public RTIF
{
public:

  const bool  DebugPrints = false;

public:

  // Parameters
  const uint32_t    NumProcThreads_FromParams = {VAL params/num_proc_threads};
  const uint32_t    NumChannels = {VAL params/num_channels};

  // Modified Parameters
  atomic<uint32_t>  NumProcThreads;
  uint32_t          MaxChannelsPerProcThread;

  // Threading and synchronization
  uint64_t  ThreadCycleCounter[{VAL params/num_proc_threads}];   // Rolling cycle counter per thread
  uint32_t  ChannelStart[{VAL params/num_proc_threads}];         // The starting channel number for each thread
  uint32_t  NumChannelsPerThread[{VAL params/num_proc_threads}]; // The number of channels a particular thread processes
  thread    Thread[{VAL params/num_proc_threads}];               // Thread handle for each thread
  thread    ControlThread;                                       // Thread handle for controller thread

  //////////////////////////////////////////////
  // Globally Available Data and Control Items
  //////////////////////////////////////////////

  // Input data  (In)
  FIFO_Reader<{VAL params/input_type},{VAL params/input_type}>     {VAL params/input_rfifo_Label};
  FifoReaderTool<{VAL params/input_type},{VAL params/input_type}>  FRT_In;

  // Output Filtered Data  (Out)
  const vector<uint32_t>  PostFilt_Dimensions;
  const vector<bool>      PostFilt_WrapFlags;
  BufMap<{VAL params/output_type},{VAL params/output_type}>  PostFilt_Extent;
  FIFO<{VAL params/output_type},{VAL params/output_type}>    {VAL params/output_fifo_label};
  FifoWriterTool<{VAL params/output_type},{VAL params/output_type}>  FWT_Out;

  //////////////////////////////////////////
  // Move to 'RenderedTemplateIF'   WEJTAG
  //////////////////////////////////////////

  bool  AnyActiveThreads(void) const;

  ////////////
  // Display
  ////////////

  string  Display_Constructor(void);
  string  Display_InitReset(void);
  string  Display_InitAlloc(void);
  string  Display_Kill(void);

public:

  {VAL label}(void);

  RTIF::ExitStateType  Init_Allocate(void);    // Called once to allocate dynamic memory
  RTIF::ExitStateType  Init_Reset(void);       // Called any time to reset all counters to their initial state
  RTIF::ExitStateType  KillThreads(void);      // Called to kill all threads

private:

  // Controller for the main processing threads
  static RTIF::ExitStateType   ProcessBatches( {VAL label}& _Obj );   // Called to process a batch of data

  // Main processing thread entry point
  static RTIF::ExitStateType   ProcessingThread( {VAL label}& _Obj, const uint32_t _ThreadNum );
};


