///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  WavFileIn.h                                                                       //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
//<LIBTAG:7a5fec18-281e-45e1-9051-9f60cc5d1cf3>

#pragma once

#include  <vector>
#include  "RTIF.h"
#include  "Wavefile.h"
#include  "FifoWriterTool.h"


class {VAL label} : public RTIF
{
public:

  const bool    DebugPrints = false;

public:

  // Parameters
  const uint32_t    NumChannels = {VAL params/num_channels};
  const string      Source_WAV_FilePath = "{ VAL params/wav_file_path }";

  // Working variables
  atomic<uint32_t>  NumProcThreads;
  WaveFileIn        InputWavFile;
  uint32_t          CurrSample;
  uint32_t          LastSample;

  // Threading
  thread            ControlThread;    // Thread handle for controller thread

  //////////////////////////////////////////////
  // Globally Available Data and Control Items
  //////////////////////////////////////////////

  // OutFifo
  const vector<uint32_t>  {VAL params/output_fifo_label}_Dimensions;
  const vector<bool>      {VAL params/output_fifo_label}_WrapFlags;

  BufMap<{VAL params/data_type},{VAL params/alloc_data_type}>   {VAL params/output_fifo_label}_Extent;
  FIFO<{VAL params/data_type},{VAL params/alloc_data_type}>     {VAL params/output_fifo_label};
  FifoWriterTool<{VAL params/data_type},{VAL params/alloc_data_type}>  FWT;

  // SourceExhausted TRUE when all samples have been delivered to the out FIFO
  atomic<bool>  SourceSamplesExhausted;

  ////////////
  // Move to 'RenderedTemplateIF'
  ////////////

  bool  AnyActiveThreads(void) const;

public:

  // Standard definitions
  {VAL label}(void);

  virtual ~{VAL label}(void)
  {
  }

  RTIF::ExitStateType  Init_Allocate(void);   // Called once to allocate dynamic memory
  RTIF::ExitStateType  Init_Reset(void);      // Called any time to reset all counters to their initial state
  RTIF::ExitStateType  KillThreads(void);     // Called to kill all threads
  RTIF::ExitStateType  Exit(void);            // Exit threads --- usually end of data

  // Display
  string      Display_ProcessBatches(void);
  string      Display_Constructor(void);
  string      Display_InitAlloc(void);
  string      Display_InitReset(void);
  string      Display_Kill(void);
  string      Display_Exit(void);

private:

  // Controller for the main processing threads
  static RTIF::ExitStateType   ProcessBatches( {VAL label}& _Obj );   // Called to process a batch of data

  // Main processing thread entry point
  static RTIF::ExitStateType   ProcessingThread( {VAL label}& _Obj, const uint32_t _ThreadNum );
};


