///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  DataPath.h                                                                        //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <string>
#include    <cassert>
#include    "GrammarObj.h"

using namespace std;


class  DataNodeRef;


class  DataPath
{
private:

  // The parser
  static wej_parser::GrammarAbs*   Parser;

  // The outputs of the parse
  wej_parser::Outputs     Out;

  // Local flags
  bool    TreeRoot;

private:

  void    ParseInPathString( const string& _PathString );

public:

  DataPath( const string& _Path = string("//") );

  DataPath    operator+( const DataPath& _Data2 ) const;

  // Node operations
  bool    DowsNodeExist() const;

  bool    IsTreeRoot(void) const
  {
    return  this->TreeRoot;
  }

  void    SetTreeRoot( const bool _NewRoot )
  {
    this->TreeRoot = _NewRoot;
  }

  void      Simplify(void);

  bool      StripAndMatchPrefix( DataPath& _Prefix );

  string    GetPathString(void) const;

  string    DisplayAll( const unsigned int Indent=0 ) const;

  inline const wej_parser::Outputs&   GetOutputs(void) const
  {
    return    Out;
  }

  inline list<wej_parser::OutputRec>&  GetOutputList(void)
  {
    return    Out.GetDataList();
  }

  static  bool    IsASyntacticallyValidPath( const string& _Path );

  friend std::ostream&   operator<<( std::ostream& _OS, const DataPath& _DP )
  {
    _OS << _DP.GetPathString();
    return  _OS;
  }
};


