///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  LiteralObj.cpp                                                                    //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include  <iostream>
#include  <iomanip>
#include  "LiteralObj.h"
#include  "CharacterObj.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// string   LiteralObj::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string    LiteralObj::Display( bool , const unsigned int  ) const
{
  ostringstream        oS;
  oS << "LIT(" << Value << ")";
  oS << DisplayFlags();
  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// bool   LiteralObj::Parse
///////////////////////////////////////////////////////////////////////////////////////////////////

bool  LiteralObj::Parse( ParseFrame PF )
{
  // Set rule
  PF.SetRule(this);

  if(GrammarAbs::DebugParse)
  {
    cout << ">>LIT  " << "[" << PF.GetRuleDebugDescriptor()
         << "]  [" << PF.GetInPos() << "," << PF.GetOutListPos() << "]  "
         << this->Display(false) << endl;
  }

  return    true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string   LiteralObj::GetOutput
///////////////////////////////////////////////////////////////////////////////////////////////////

string    LiteralObj::GetOutput( ParseFrame& PF, const unsigned int RuleNum )
{
  UNUSED(PF);
  UNUSED(RuleNum);

  return    this->Value;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// void   LiteralObj::GenerateCppParseTree
///////////////////////////////////////////////////////////////////////////////////////////////////

void    LiteralObj::GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes )
{
  ostringstream     oS;
  UNUSED(Includes);

  oS << "  LIT = new LiteralObj;" << endl;

  if( KleinLayer )
  {
    oS << "  KLEIN->SetKleinChild(LIT);" << endl;
  }
  else
  {
    oS << "  AND->PushBack(LIT);" << endl;
  }

  oS << "  LIT->SetValue(\"" << CharacterObj::DisplayCharCpp( GetValue() ) << "\");" << endl;

  // Accumulate the output
  OutputData.Subroutines.back().Contents  += oS.str();
}
}   // End namespace 'wej_parser'


