///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  Outputs.h                                                                         //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <cassert>
#include    <string>
#include    <vector>
#include    <list>

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class    OutputRec
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class   OutputRec
{
public:

  bool              Debug;
  vector<string>    Data;

public:

  inline  OutputRec(void) : Debug(false)
  {
  }

  const OutputRec&  operator=( const OutputRec& _OutputRec )
  {
    Debug = _OutputRec.Debug;
    Data = _OutputRec.Data;

    return  *this;
  }

  inline  OutputRec( const string pLabel, const bool pDebug = false ) : Debug(pDebug)
  {
    Data.push_back(pLabel);
  }

  inline  OutputRec( const OutputRec& OR ) : Debug(OR.Debug), Data(OR.Data)
  {
  }

  inline  vector<string>&   GetDataVect(void)
  {
    return    Data;
  }

  inline  const vector<string>&   GetDataVect(void) const
  {
    return    Data;
  }

  inline bool   IsDebug(void) const
  {
    return    Debug;
  }

  //////////////////////////////////////////
  // Display
  //////////////////////////////////////////

  string    Display( const unsigned int Indent=0 ) const;
};


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class  Outputs
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class   Outputs
{
private:

  list<OutputRec>   Data;

public:

  Outputs(void)
  {
  }

  inline Outputs( const Outputs& _Other )
  {
    Data  = _Other.Data;
  }

  const Outputs   operator=( const Outputs& _Outputs )
  {
    Data = _Outputs.Data;

    return  *this;
  }

  inline list<OutputRec>::size_type   NumRecords(void) const
  {
    return  Data.size();
  }

  inline void     Clear(void)
  {
    Data.clear();
  }

  //////////////////////////////////////////
  // List Access Iterator Edges
  //////////////////////////////////////////

  inline list<OutputRec>::const_iterator     Begin(void) const
  {
    return    Data.begin();
  }

  inline list<OutputRec>::const_iterator     End(void) const
  {
    return    Data.end();
  }

  inline list<OutputRec>::iterator           Begin(void)
  {
    return    Data.begin();
  }

  inline list<OutputRec>::iterator           End(void)
  {
    return    Data.end();
  }

  inline const OutputRec&                    Front(void) const
  {
    return    Data.front();
  }

  inline const OutputRec&                    Back(void) const
  {
    return    Data.back();
  }

  inline OutputRec&                          Front(void)
  {
    return    Data.front();
  }

  inline OutputRec&                          Back(void)
  {
    return    Data.back();
  }

  //////////////////////////////////////////
  // Display
  //////////////////////////////////////////

  string    Display( const unsigned int Indent=0 ) const;

  //////////////////////////////////////////
  // List Access --- Whole
  //////////////////////////////////////////

  inline  list<OutputRec>&        GetDataList(void)
  {
    return    Data;
  }

  inline  const list<OutputRec>&  GetDataList(void) const
  {
    return    Data;
  }
};
}   // End namespace 'wej_parser'


