///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  ParseFrame.h                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include      <typeinfo>
#include      <cassert>
#include      <iostream>
#include      <list>
#include      <vector>
#include      <string>
#include      <ios>
#include      "Outputs.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class      ParseFrame
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class     GrammarAbs;
class     AndRuleObj;


class     ParseFrame
{
private:

  istream&      IN;
  Outputs&      ListOut;
  bool          DebugOutput;
  ParseFrame*   Father;
  GrammarAbs*   Rule;

  // Used to store the start character number in the input stream.  These values
  //    are used when forming the output strings.  Note that there is an extra
  //    element at the end.  Note also that this vector is ONLY used for 'ParseFrame's
  //    corresponding to 'AndRuleObj's.
  vector<streampos>             StartInputCharPos;    // Element # + 1

public:

  streampos                     PosIn;                // On Mark
  list<OutputRec>::size_type    PosLOut;              // On Mark
  vector<string>::size_type     PosL2Out;             // On Mark
  string::size_type             PosL2SOut;            // On Mark

  streampos                     PosIn0;               // On Entry
  list<OutputRec>::size_type    PosLOut0;             // On Entry
  vector<string>::size_type     PosL2Out0;            // On Entry
  string::size_type             PosL2SOut0;           // On Entry

protected:

  inline void     MarkIn(void)
  {
    this->PosIn   = this->GetInPos();
  }

  inline void     MarkOutList(void)
  {
    this->PosLOut     = this->GetOutListPos();
    this->PosL2Out    = this->GetOutList2Pos();
    this->PosL2SOut   = this->GetOutList2SPos();
  }

  inline void     Mark(void)
  {
    this->MarkIn();
    this->MarkOutList();
  }

  inline void      MarkFirst(void)
  {
    this->PosIn0      = this->PosIn;
    this->PosLOut0    = this->PosLOut;
    this->PosL2Out0   = this->PosL2Out;
    this->PosL2SOut0  = this->PosL2SOut;
  }

  void            OutputMatchedList(void);

  inline void     MoveInFileTo( streampos  NewPos )
  {
    this->IN.clear();
    this->IN.seekg( NewPos, ios::beg );
  }

  void  MoveOutListTo
        (
          list<OutputRec>::size_type    NewPos,
          vector<string>::size_type     NewPos2,
          string::size_type             NewPos2S
        );

public:

  inline  ParseFrame( istream& pIn, Outputs& pOutList, bool pDebugOutput=false ) :
                      IN(pIn), ListOut(pOutList), DebugOutput(pDebugOutput),
                      Father(nullptr), Rule(nullptr)
  {
    Mark();
    MarkFirst();
  }

  inline  ParseFrame( ParseFrame& Other ) :
                      IN(Other.IN), ListOut(Other.ListOut), DebugOutput(Other.DebugOutput),
                      Father(&Other), Rule(nullptr)
  {
    Mark();
    MarkFirst();
  }

  ~ParseFrame(void);

  inline ParseFrame*      GetFather(void)
  {
    return  this->Father;
  }

  void    SetRule( GrammarAbs* pRule );

  // At this level
  string                GetSegmentChars( const unsigned int SegmentNum );

  inline GrammarAbs*    GetRule(void)
  {
    return    this->Rule;
  }

  inline void   AcceptAll(void)
  {
    this->Mark();
  }

  inline void   AcceptIn(void)
  {
    this->MarkIn();
  }

  inline void   AcceptOutList(void)
  {
    this->MarkOutList();
  }

  inline bool   IsDebug(void) const
  {
    return    this->DebugOutput;
  }

  // Returns the number of characters actual read
  unsigned int    ReadFromInput( const unsigned int Length, string& Result ) const;

  // Returns the number of characters actual read
  inline unsigned int   ReadCharFromInput( char& Result ) const
  { // Read the data
    IN.read( &Result, 1 );
    return      IN.gcount();
  }

  inline bool   IsInEof(void) const
  {
    return    IN.eof();
  }

  inline streampos   GetInPos(void) const
  {
    return    IN.tellg();
  }

  inline  list<OutputRec>::size_type  GetOutListPos(void) const
  {
    return    ListOut.GetDataList().size();
  }

  vector<string>::size_type   GetOutList2Pos(void) const;

  string::size_type   GetOutList2SPos(void) const;

  inline bool   IsDebugOutput(void) const
  {
    return    DebugOutput;
  }

  void  SaveInputFilePosition( const unsigned int StatementRuleNum );

  // If 'RuleName' is not specified then the next AndRuleObj up is returned
  AndRuleObj*   FindUpperAndRule( ParseFrame** PF_AtAnd, const string RuleName = string() );

  // The PF corresponding to the GrammarObj
  bool    IsTopmost(void) const;

  //////////////////////////////////
  // Output List Specific Function
  //////////////////////////////////

  // Push a new record (vector<string>) on the OutList
  inline void   PushNewOutListRecord(void)
  {
    ListOut.GetDataList().push_back( OutputRec() );
  }

  // Get a reference to the last element in the OutList
  inline OutputRec&   EndOutListRec(void)
  {
    assert( ListOut.GetDataList().size() > 0 );
    return    ListOut.GetDataList().back();
  }

  // Get a reference to the last element in the OutList
  inline vector<string>&  EndOutListRecStrings(void)
  {
    assert( ListOut.GetDataList().size() > 0 );
    return    ListOut.GetDataList().back().GetDataVect();
  }

  // Get rule info for debug
  string    GetRuleDebugDescriptor(void) const;

  //////////////////////////////////
  // Output List Display
  //////////////////////////////////

  string    Display(void) const;
  string    Display2(void) const;
  string    Display3(void) const;

  static string   Display( const Outputs&   OutList );
  static string   Display( const OutputRec& OutRec  );
};
}   // End namespace 'wej_parser'


