///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  FirFilterFourierCoefs_001_lab.h                                                   //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
//<LIBTAG:de887803-6dbf-46a0-9373-29a0053d9adb>

#pragma once

#include  "AlignedMemAlloc.h"
#include  "RTIF.h"
#include  "LS_FIR_Filter.h"
#include  "FifoWriterTool.h"


class FirFilterFourierCoefs_001_lab : public RTIF
{
public:

  const bool    DebugPrints = false;

  typedef struct _FilterSegDef
  {
    LS_FIR_Filter::SegType  Type;
    bool                    Symmetric;
    double                  F0;
    double                  F1;
    double                  A0;
    double                  A1;
    double                  W;
  }   FilterSegDef;

  typedef struct _FilterDef
  {
     vector<FilterSegDef>  FilterSeg;
  }   FilterDef;

public:

  // Parameters
  const uint32_t  NumProcThreads_FromParams = 2;
  const uint32_t  NumChannels = 2;
  const uint32_t  FirFilterFreqDomainCoefsLen = 1024;
  const uint32_t  NumFilterCoefs = 513;
  const double    SampleRate = 48000.;

  // Filter Definition
  static const vector<FilterDef>  FilterDefs;

  // Filter coefficients solution
  LS_FIR_Filter     FilterSolutions[2];

  // Modified Parameters
  atomic<uint32_t>  NumProcThreads;
  uint32_t          MaxChannelsPerProcThread;

  // Threading and synchronization
  uint64_t          ThreadCycleCounter[2];   // Rolling cycle counter per thread
  uint32_t          ChannelStart[2];         // The starting channel number for each thread
  uint32_t          NumChannelsPerThread[2]; // The number of channels a particular thread processes
  thread            Thread[2];               // Thread handle for each thread
  thread            ControlThread;                                       // Thread handle for controller thread

  //////////////////////////////////////////////
  // Globally Available Data and Control Items
  //////////////////////////////////////////////

  // OutFifo
  const vector<uint32_t>  CoefsFifo_Dimensions;
  const vector<bool>      CoefsFifo_WrapFlags;

  BufMap<complex<double>,complex<double>>  CoefsFifo_Extent;
  FIFO<complex<double>,complex<double>>  CoefsFifo;
  FifoWriterTool<complex<double>,complex<double>>  FWT;

  ///////////////////////////////////////////////
  // Information needed from filter generation
  ///////////////////////////////////////////////

  atomic<bool>  FilterGenerationComplete;

  uint32_t  GetNumFilterCoefs(void) const
  {
     return  this->NumFilterCoefs;
  }

  uint32_t  GetFftLen(void) const
  {
    return  1024;
  }

  ////////////
  // Display
  ////////////

  string      Display_Constructor(void);
  string      Display_InitReset(void);
  string      Display_Kill(void);
  string      Display_Exit(void);

public:

  FirFilterFourierCoefs_001_lab(void);

  RTIF::ExitStateType  Init_Allocate(void);    // Called once to allocate dynamic memory
  RTIF::ExitStateType  Init_Reset(void);       // Called any time to reset all counters to their initial state
  RTIF::ExitStateType  KillThreads(void);      // Called to kill all threads
  RTIF::ExitStateType  Exit(void);             // Called to exit all threads

private:

  // Controller for the main processing threads
  static RTIF::ExitStateType   ProcessBatches( FirFilterFourierCoefs_001_lab& _Obj );   // Called to process a batch of data

  // Main processing thread entry point
  static RTIF::ExitStateType   ProcessingThread( FirFilterFourierCoefs_001_lab& _Obj, const uint32_t _ThreadNum );
};


