///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  start.cpp                                                                         //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
/*##################################################################################################
## Data Definition: ''
## Source: ''
## Rendered by 'xml_decorator' at 20231109_224856.
## Copyright 2023, William Earl Jones, All Rights Reserved    [info: http://www.wejc.com]
##################################################################################################*/

// Module includes
#include  <fftw3.h>
#include  <mutex>
#include  <malloc.h>
#include  "modules.h"
#include  "Exception.h"

mutex  FFTW_Mut;

int    main( int /* NumArgs */, char** /* ArgList */ )
{
  assert( mallopt( M_CHECK_ACTION, 0 ) == 1 );

  try
  {
    {
      lock_guard<mutex>  Lock(FFTW_Mut);
      auto Flag = fftw_init_threads();
      if( !Flag )
      {
        Exception  E(__FILE__,__LINE__);
        E.Display() = "Error executing fftw_init_threads().";
      }
    }

  ///////////////////////////
  // Init_Allocate
  ///////////////////////////

  WAV_File_In_001_lab_dec.Init_Allocate();
  FirFilterFourierCoefs_001_lab_dec.Init_Allocate();
  OverlapAndAddFiltering_001_lab_dec.Init_Allocate();
  WAV_File_Out_001_lab_dec.Init_Allocate();

  ///////////////////////////
  // Init_Reset
  ///////////////////////////

  WAV_File_In_001_lab_dec.Init_Reset();
  FirFilterFourierCoefs_001_lab_dec.Init_Reset();
  OverlapAndAddFiltering_001_lab_dec.Init_Reset();
  WAV_File_Out_001_lab_dec.Init_Reset();

  cout << endl << "=====>  STARTING REAL-TIME OPERATION  <=====" << endl << endl;
  cout.flush();

  //////////////////////////////
  // Debug sleep in background
  //////////////////////////////

  std::this_thread::sleep_for(std::chrono::seconds(14s));

  DebugMutex.lock();
  cout << endl << "=====> Got Kill <=====" << endl;
  DebugMutex.unlock();

  ///////////////////////////
  // KillThreads
  ///////////////////////////

  WAV_File_In_001_lab_dec.KillThreads();
  FirFilterFourierCoefs_001_lab_dec.KillThreads();
  OverlapAndAddFiltering_001_lab_dec.KillThreads();
  WAV_File_Out_001_lab_dec.KillThreads();

  DebugMutex.lock();
  cout << endl << 
    "===> All the threads are dead and joined back up --- Program Terminated Normally " 
    "<===" << endl << endl;
  DebugMutex.unlock();
  }

  catch(Exception&  DE)
  {
    cout << endl << DE.Display() << endl << endl;
    return  1;
  }

  return  0;
}

