///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  GenerateFiles.h                                                                   //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include  <list>
#include  <string>
#include  <sstream>
#include  <chrono>
#include  <filesystem>
#include  "DataRef.h"
#include  "DataNodeRef.h"
#include  "TemplateRenderer.h"

using namespace std;
namespace fs = filesystem;

///////////////////////////////////////////////////////////////////////////////////////////////////
// class   RefRec
///////////////////////////////////////////////////////////////////////////////////////////////////

class   RefRec
{
public:

  RefRec( string ReferencingModule ) : ModuleName(ReferencingModule)
  {
  }

  string    GetModuleName(void) const
  {
    return  this->ModuleName;
  }

  void      AddReferencedModule( string A_ReferencedModule )
  {
    ReferencedModules.push_back(A_ReferencedModule);
  }

  list<string>&   ReferencesList(void)
  {
    return  this->ReferencedModules;
  }

  string    Display(void) const;

  static string   DisplayList( list<RefRec>& List );

  // Reorder list so reference modules come first
  static void    ReorderList( list<RefRec>& ModuleList );

private:

  string          ModuleName;
  list<string>    ReferencedModules;
};


///////////////////////////////////////////////////////////////////////////////////////////////////
// TemplateRenderer
///////////////////////////////////////////////////////////////////////////////////////////////////

extern  TemplateRenderer    Renderer;
extern  list<RefRec>        Modules;

///////////////////////////////////////////////////////////////////////////////////////////////////
// Misc files used to generate the output files
///////////////////////////////////////////////////////////////////////////////////////////////////

void    Init_StartCPP_File( const string& OutputPath, const string& NowTime );
void    Init_ModulesH_File( const string& OutputPath, const string& NowTime );
void    Init_ModulesCPP_File( const string& OutputPath, const string& NowTime );


void  ProcessTemplates
      (
        DataNodeRef     ModuleDef,
        const string&   OutputPath,
        const string&   NowTime
      );


void    ProcessTemplate
        (
          const fs::path&  LibPath,
          const fs::path&  OutPath,
          DataNodeRef      ProgramDef,          // PROGRAM.XML being processed
          DataNodeRef      LibManifestRecDef,   // LIB_MANIFEST.XML: all files for uuid
          DataNodeRef      LibManifestFileDef,  // LIB_MANIFEST.XML: a particular file
          const string&    NowTime
        );


string  CreateTimestamp
        (
          const string&   NowTime,
          const string    Source = "",
          const string    Definition = ""
        );


DataPath  LookupTagFromLabel( const string& LabelParam );


