///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  GrammarObj.h                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include  <cassert>
#include  <iostream>
#include  <vector>
#include  <map>
#include  "GrammarAbs.h"
#include  "ParseFrame.h"
#include  "InverseParseFrame.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class  GrammarObj : public GrammarAbs
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class   OrRuleObj;


class   GrammarObj : public GrammarAbs
{
public:

  static const bool     ParseTime = false;

  typedef struct sRuleLoc
  {
    unsigned int      RuleNum;
    GrammarAbs*       Address;
  }   MapRecType;

  // Output tokens on list after parse
  enum    eTokens
  {
    PARSER_Token,
    XEQ_Token,
    PUMP_Token,
    STMNT_Token,
    AND_Token,
    REF_CMND_Token,
    KLEIN_Token,
    OUT_LIST_START_Token,
    OUT_LIST_A_ITEM_Token,
    OUT_LIST_A_CHAR_Token,
    OUT_LIST_EL_Token,
    CHAR_START_Token,
    CHAR_Token,
    CHAR_SINGLE_Token,
    CHAR_RANGE_Token,
    CHAR_ALL_Token,
    CHAREOF_Token,
    CHAR_NOT_END_Token,
    HEXCHAR_Token,
    LITERAL_Token,
    LIT_START_Token,
    LIT_EL_Token,
    LIT_HEX2_Token,
    LIT_END_Token,
    TAG_Token
  };

private:

  vector<GrammarAbs*>               Group;
  map<const string,MapRecType>      Rules;
  bool                              IsPump;

  // Throughput estimation
  double                            ParseTimeSecs;
  unsigned long int                 ParseChars;
  double                            ParseCharsPerSecond;

  // Token --- Rule Num Map
  static const map<string,int>      TokenToRuleMap;

public:

  GrammarObj(void) : IsPump(false), ParseTimeSecs(0.), ParseChars(0), ParseCharsPerSecond(0.)
  {
  }

  virtual ~GrammarObj();

  inline const map<const string,MapRecType>&    GetRuleMap(void) const
  {
    return    Rules;
  }

  inline void   PushBack( GrammarAbs* Rule )
  {
    Group.push_back(Rule);
    Rule->SetFather(this);

    MapRecType    MR;
    MR.RuleNum    = Group.size();
    MR.Address    = Group.back();

    // Save a map entry
    Rules.insert( pair<const string,MapRecType>( Rule->GetLabel(), MR ) );
  }

  // Look-up rule by label
  OrRuleObj*    LookUpRuleByLabel( const string LabelToLookup );

  // Generate a memory BNF tree from output list of BNFC parse
  static GrammarObj*    CreateGrammarFromBnfcSource( istream& In );

  // IsPump field
  void      SetIsPump( const bool NewSetting )
  {
    this->IsPump  = NewSetting;
  }

  bool      GetIsPump(void) const
  {
    return    this->IsPump;
  }

  bool      ParseRule( GrammarAbs* StartRule, ParseFrame& PF );
  bool      ParseRule( const string& StartRuleLabel, ParseFrame& PF );

  bool      ParseRule( const unsigned int StartRuleNum, ParseFrame& PF )
  {
    assert( this->Group.size() > StartRuleNum );
    return    ParseRule( this->Group[StartRuleNum], PF );
  }

  //////////////////////////////
  // Interfaces
  //////////////////////////////

  string          Display( bool SubRule=false, const unsigned int Inden=0 ) const;

  bool            Parse( ParseFrame PF )
  {
    assert(!this->Group.empty());

    return    ParseRule( this->Group[0], PF );
  }

  string          GetOutput( ParseFrame& PF, const unsigned int RuleNum );
  void            GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes );

  inline bool     DoesRead(void) const
  {
    return    true;
  }

  bool            InverseParse( InverseParseFrame IPF );
};
}   // End namespace 'wej_parser'


