///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  Translator.h                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    "BNFC_Gen.h"
#include    "GrammarObj.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
//  class   Translator
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class   Translator
{
private:

  GrammarObj*   TransGram;

public:

  inline  Translator(void) : TransGram( dynamic_cast<GrammarObj*>(wej_parser::wej_bnf::Parse_BNFC()) )
  {
  }

  inline  Translator( istream& BnfcSource )
  {
    TransGram   = GrammarObj::CreateGrammarFromBnfcSource( BnfcSource );
  }

  inline  ~Translator(void)
  {
    if( TransGram != dynamic_cast<GrammarObj*>(wej_parser::wej_bnf::Parse_BNFC()) )   delete TransGram;
  }

  inline GrammarObj*  GetGrammarObj(void)
  {
    return    TransGram;
  }

  inline bool   IsGrammarGood(void) const
  {
    return    TransGram != nullptr;
  }

  bool    Translate( istream& In, Outputs& OutList );

  // Using hard-coded BNF translator in 'BNFC_Gen'
  static bool   TranslateFromHardCodedBnfc( istream& In, Outputs& OutList );

  static GrammarObj*    GetIBnfFromHardCodedBnfc  ( istream& In )
  {
    return  GrammarObj::CreateGrammarFromBnfcSource( In );
  }
};
}   // End namespace 'wej_parser'


